@@ -25,11 +25,18 @@ fallback can be optimized to no fallback variants.
 from nuitka import Variables
 from nuitka.optimizations.TraceCollections import TraceCollectionBranch
 from nuitka.PythonVersions import python_version
+from nuitka.tree.TreeHelpers import makeStatementsSequence
 
+from .AssignNodes import (
+    StatementAssignmentVariable,
+    StatementDelVariable,
+    StatementReleaseVariable
+)
 from .ConditionalNodes import ExpressionConditional
 from .ConstantRefNodes import ExpressionConstantDictEmptyRef
 from .ExpressionBases import ExpressionBase, ExpressionChildrenHavingBase
 from .NodeBases import StatementBase, StatementChildHavingBase
+from .VariableRefNodes import ExpressionTempVariableRef
 
 
 class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
@@ -155,7 +162,7 @@ class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
             )
 
             # Make the original one for the fallback
-            call_node = call_node.makeClone()
+            call_node = call_node.makeCloneShallow()
             call_node.setCalled(self.subnode_fallback)
 
             result = ExpressionConditional(
@@ -200,6 +207,11 @@ class ExpressionLocalsVariableRef(ExpressionBase):
 
         self.variable_trace = None
 
+    def finalize(self):
+        del self.parent
+        del self.locals_scope
+        del self.variable
+
     def getDetails(self):
         return {
             "variable_name" : self.getVariableName(),
@@ -218,6 +230,30 @@ class ExpressionLocalsVariableRef(ExpressionBase):
         return self.locals_scope
 
     def computeExpressionRaw(self, trace_collection):
+        if self.locals_scope.isMarkedForPropagation():
+            variable_name = self.getVariableName()
+
+            variable = self.locals_scope.allocateTempReplacementVariable(
+                trace_collection = trace_collection,
+                variable_name    = variable_name
+            )
+
+            result = ExpressionTempVariableRef(
+                variable   = variable,
+                source_ref = self.source_ref
+            )
+            result.parent = self.parent
+
+            self.finalize()
+
+            new_result = result.computeExpressionRaw(trace_collection)
+
+            if new_result[0] is not result:
+                assert False, (new_result, result)
+
+            return result, "new_expression", "Replaced dictionary ref with temporary variable."
+
+
         self.variable_trace = trace_collection.getVariableCurrentTrace(
             variable = self.variable
         )
@@ -270,6 +306,10 @@ class ExpressionLocalsVariableCheck(ExpressionBase):
 
         self.locals_scope = locals_scope
 
+    def finalize(self):
+        del self.parent
+        del self.locals_scope
+
     def getDetails(self):
         return {
             "locals_scope"  : self.locals_scope,
@@ -283,6 +323,7 @@ class ExpressionLocalsVariableCheck(ExpressionBase):
         return self.locals_scope
 
     def computeExpressionRaw(self, trace_collection):
+        assert not self.locals_scope.isMarkedForPropagation()
         return self, None, None
 
 
@@ -317,9 +358,14 @@ class StatementLocalsDictOperationSet(StatementChildHavingBase):
 
         self.locals_scope = locals_scope
 
+    def finalize(self):
+        del self.parent
+        del self.locals_scope
+        del self.variable_trace
 
     def getDetails(self):
         return {
+            "locals_scope"  : self.locals_scope,
             "variable_name" : self.getVariableName()
         }
 
@@ -329,7 +375,33 @@ class StatementLocalsDictOperationSet(StatementChildHavingBase):
     def getLocalsDictScope(self):
         return self.locals_scope
 
+    # TODO: Child name inconsisten with accessor name.
+    getAssignSource = StatementChildHavingBase.childGetter("value")
+
     def computeStatement(self, trace_collection):
+        if self.locals_scope.isMarkedForPropagation():
+            variable_name = self.getVariableName()
+
+            variable = self.locals_scope.allocateTempReplacementVariable(
+                trace_collection = trace_collection,
+                variable_name    = variable_name
+            )
+
+            result = StatementAssignmentVariable(
+                source     = self.subnode_value,
+                variable   = variable,
+                source_ref = self.source_ref
+            )
+            result.parent = self.parent
+
+            new_result = result.computeStatement(trace_collection)
+
+            if new_result[0] is not result:
+                assert False, (new_result, result)
+
+            self.finalize()
+            return result, "new_statements", "Replaced dictionary assigment with temporary variable."
+
         result, change_tags, change_desc = self.computeStatementSubExpressions(
             trace_collection = trace_collection
         )
@@ -363,7 +435,6 @@ class StatementLocalsDictOperationDel(StatementBase):
     # TODO: Specialize for Python3 maybe to save attribute for Python2.
     may_raise_del = python_version >= 300
 
-
     def __init__(self, locals_scope, variable_name, source_ref):
         assert type(variable_name) is str
 
@@ -381,6 +452,12 @@ class StatementLocalsDictOperationDel(StatementBase):
 
         self.previous_trace = None
 
+    def finalize(self):
+        del self.parent
+        del self.locals_scope
+        del self.variable
+        del self.previous_trace
+
     def getDetails(self):
         return {
             "variable_name" : self.getVariableName(),
@@ -394,6 +471,28 @@ class StatementLocalsDictOperationDel(StatementBase):
         return self.locals_scope
 
     def computeStatement(self, trace_collection):
+        if self.locals_scope.isMarkedForPropagation():
+            variable_name = self.getVariableName()
+
+            variable = self.locals_scope.allocateTempReplacementVariable(
+                trace_collection = trace_collection,
+                variable_name    = variable_name
+            )
+
+            result = StatementDelVariable(
+                variable   = variable,
+                tolerant   = False,
+                source_ref = self.source_ref
+            )
+            result.parent = self.parent
+
+            new_result = result.computeStatement(trace_collection)
+
+            if new_result[0] is not result:
+                assert False, (new_result, result)
+
+            return result, "new_statements", "Replaced dictionary del with temporary variable."
+
         self.previous_trace = trace_collection.getVariableCurrentTrace(self.variable)
 
         # The "del" is a potential use of a value. TODO: This could be made more
@@ -440,6 +539,11 @@ class StatementSetLocals(StatementChildHavingBase):
 
         self.locals_scope = locals_scope
 
+    def finalize(self):
+        del self.parent
+        del self.locals_scope
+        del self.subnode_new_locals
+
     def getDetailsForDisplay(self):
         return {
             "locals_scope" : self.locals_scope.getCodeName()
@@ -457,6 +561,7 @@ class StatementSetLocals(StatementChildHavingBase):
         return self.getNewLocals().mayRaiseException(exception_type)
 
     getNewLocals = StatementChildHavingBase.childGetter("new_locals")
+    getAssignSource = getNewLocals
 
     def computeStatement(self, trace_collection):
         trace_collection.onExpression(self.getNewLocals())
@@ -474,6 +579,12 @@ class StatementSetLocals(StatementChildHavingBase):
             return result, "new_raise", """\
 Setting locals already raises implicitly building new locals."""
 
+        if self.locals_scope.isMarkedForPropagation():
+            self.finalize()
+
+            return None, "new_statements", """\
+Forward propagating locals."""
+
         return self, None, None
 
     def getStatementNiceName(self):
@@ -513,7 +624,31 @@ class StatementReleaseLocals(StatementBase):
 
         self.locals_scope = locals_scope
 
+    def finalize(self):
+        del self.parent
+        del self.locals_scope
+
     def computeStatement(self, trace_collection):
+        if self.locals_scope.isMarkedForPropagation():
+            statements = [
+                StatementReleaseVariable(
+                    variable   = variable,
+                    source_ref = self.source_ref
+                )
+                for variable
+                in self.locals_scope.getPropagationVariables().values()
+            ]
+
+            result = makeStatementsSequence(
+                statements = statements,
+                allow_none = False,
+                source_ref = self.source_ref
+            )
+
+            self.finalize()
+
+            return result, "new_statements", "Releasing temp variables instead of locals dict."
+
         return self, None, None
 
     def getDetails(self):
