@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -34,8 +34,8 @@ from nuitka.nodes.SliceNodes import (
 from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
 from nuitka.PythonVersions import python_version
 
-from .Helpers import buildNode, getKind
 from .ReformulationAssignmentStatements import buildExtSliceNode
+from .TreeHelpers import buildNode, getKind
 
 
 def buildSubscriptNode(provider, node, source_ref):
